var Tree = require('..').RBTree;

// create a new tree, pass in the compare function
var tree = new Tree(function(a, b) { return a - b; });

// do some inserts
tree.insert(1);
tree.insert(2);
tree.insert(3);
tree.remove(2);

// get smallest item
tree.min();
